@@ -17,7 +17,9 @@ use std::{marker::Unpin, path::Path, string::FromUtf8Error, sync::Arc};
 use tokio::{
     io::{self, AsyncRead, AsyncWrite},
     process::Command,
+    runtime::{Handle, Runtime},
     sync::{broadcast, mpsc, oneshot, Mutex},
+    time::Duration,
 };
 
 #[derive(Debug, Display, Error, From)]
@@ -39,13 +41,16 @@ struct ConnState {
 }
 
 pub fn run(cmd: LaunchSubcommand, opt: CommonOpt) -> Result<(), Error> {
-    let rt = tokio::runtime::Runtime::new()?;
+    let rt = Runtime::new()?;
     let session_output = cmd.session;
     let mode = cmd.mode;
     let is_daemon = cmd.daemon;
 
     let session_file = cmd.session_data.session_file.clone();
     let session_socket = cmd.session_data.session_socket.clone();
+    let fail_if_socket_exists = cmd.fail_if_socket_exists;
+    let timeout = opt.to_timeout_duration();
+    let shutdown_after = cmd.to_shutdown_after_duration();
 
     let session = rt.block_on(async { spawn_remote_server(cmd, opt).await })?;
 
@@ -57,7 +62,7 @@ pub fn run(cmd: LaunchSubcommand, opt: CommonOpt) -> Result<(), Error> {
         }
         SessionOutput::Keep => {
             debug!("Entering interactive loop over stdin");
-            rt.block_on(async { keep_loop(session, mode).await })?
+            rt.block_on(async { keep_loop(session, mode, timeout).await })?
         }
         SessionOutput::Pipe => {
             debug!("Piping session to stdout");
@@ -78,8 +83,17 @@ pub fn run(cmd: LaunchSubcommand, opt: CommonOpt) -> Result<(), Error> {
                     // NOTE: We need to create a runtime within the forked process as
                     //       tokio's runtime doesn't support being transferred from
                     //       parent to child in a fork
-                    let rt = tokio::runtime::Runtime::new()?;
-                    rt.block_on(async { socket_loop(session_socket, session).await })?
+                    let rt = Runtime::new()?;
+                    rt.block_on(async {
+                        socket_loop(
+                            session_socket,
+                            session,
+                            timeout,
+                            fail_if_socket_exists,
+                            shutdown_after,
+                        )
+                        .await
+                    })?
                 }
                 Ok(_) => {}
                 Err(x) => return Err(Error::ForkError(x)),
@@ -91,7 +105,16 @@ pub fn run(cmd: LaunchSubcommand, opt: CommonOpt) -> Result<(), Error> {
                 "Entering interactive loop over unix socket {:?}",
                 session_socket
             );
-            rt.block_on(async { socket_loop(session_socket, session).await })?
+            rt.block_on(async {
+                socket_loop(
+                    session_socket,
+                    session,
+                    timeout,
+                    fail_if_socket_exists,
+                    shutdown_after,
+                )
+                .await
+            })?
         }
         #[cfg(not(unix))]
         SessionOutput::Socket => {
@@ -106,9 +129,9 @@ pub fn run(cmd: LaunchSubcommand, opt: CommonOpt) -> Result<(), Error> {
     Ok(())
 }
 
-async fn keep_loop(session: Session, mode: Mode) -> io::Result<()> {
+async fn keep_loop(session: Session, mode: Mode, duration: Duration) -> io::Result<()> {
     use crate::cli::subcommand::action::inner;
-    match Client::tcp_connect(session).await {
+    match Client::tcp_connect_timeout(session, duration).await {
         Ok(client) => {
             let config = match mode {
                 Mode::Json => inner::LoopConfig::Json,
@@ -121,11 +144,17 @@ async fn keep_loop(session: Session, mode: Mode) -> io::Result<()> {
 }
 
 #[cfg(unix)]
-async fn socket_loop(socket_path: impl AsRef<Path>, session: Session) -> io::Result<()> {
+async fn socket_loop(
+    socket_path: impl AsRef<Path>,
+    session: Session,
+    duration: Duration,
+    fail_if_socket_exists: bool,
+    shutdown_after: Option<Duration>,
+) -> io::Result<()> {
     // We need to form a connection with the actual server to forward requests
     // and responses between connections
     debug!("Connecting to {} {}", session.host, session.port);
-    let mut client = Client::tcp_connect(session).await?;
+    let mut client = Client::tcp_connect_timeout(session, duration).await?;
 
     // Get a copy of our client's broadcaster so we can have each connection
     // subscribe to it for new messages filtered by tenant
@@ -141,56 +170,82 @@ async fn socket_loop(socket_path: impl AsRef<Path>, session: Session) -> io::Res
                 "Forwarding request of type {} to server",
                 req.payload.as_ref()
             );
-            if let Err(x) = client.fire(req).await {
+            if let Err(x) = client.fire_timeout(req, duration).await {
                 error!("Client failed to send request: {:?}", x);
                 break;
             }
         }
     });
 
+    // Remove the socket file if it already exists
+    if !fail_if_socket_exists && socket_path.as_ref().exists() {
+        debug!("Removing old unix socket instance");
+        tokio::fs::remove_file(socket_path.as_ref()).await?;
+    }
+
     // Continue to receive connections over the unix socket, store them in our
     // connection mapping
     debug!("Binding to unix socket: {:?}", socket_path.as_ref());
     let listener = tokio::net::UnixListener::bind(socket_path)?;
 
-    while let Ok((conn, _)) = listener.accept().await {
-        // Create a unique id to associate with the connection since its address
-        // is not guaranteed to have an identifiable string
-        let conn_id: usize = rand::random();
+    let (ct, notify) = utils::new_shutdown_task(Handle::current(), shutdown_after);
 
-        // Establish a proper connection via a handshake, discarding the connection otherwise
-        let transport = match Transport::from_handshake(conn, None).await {
-            Ok(transport) => transport,
-            Err(x) => {
-                error!("<Client @ {:?}> Failed handshake: {}", conn_id, x);
-                continue;
+    loop {
+        tokio::select! {
+            result = listener.accept() => {match result {
+                Ok((conn, _)) => {
+                    // Create a unique id to associate with the connection since its address
+                    // is not guaranteed to have an identifiable string
+                    let conn_id: usize = rand::random();
+
+                    // Establish a proper connection via a handshake, discarding the connection otherwise
+                    let transport = match Transport::from_handshake(conn, None).await {
+                        Ok(transport) => transport,
+                        Err(x) => {
+                            error!("<Client @ {:?}> Failed handshake: {}", conn_id, x);
+                            continue;
+                        }
+                    };
+                    let (t_read, t_write) = transport.into_split();
+
+                    // Used to alert our response task of the connection's tenant name
+                    // based on the first
+                    let (tenant_tx, tenant_rx) = oneshot::channel();
+
+                    // Create a state we use to keep track of connection-specific data
+                    debug!("<Client @ {}> Initializing internal state", conn_id);
+                    let state = Arc::new(Mutex::new(ConnState::default()));
+
+                    // Spawn task to continually receive responses from the client that
+                    // may or may not be relevant to the connection, which will filter
+                    // by tenant and then along any response that matches
+                    let res_rx = broadcaster.subscribe();
+                    let state_2 = Arc::clone(&state);
+                    tokio::spawn(async move {
+                        handle_conn_outgoing(conn_id, state_2, t_write, tenant_rx, res_rx).await;
+                    });
+
+                    // Spawn task to continually read requests from connection and forward
+                    // them along to be sent via the client
+                    let req_tx = req_tx.clone();
+                    let ct_2 = Arc::clone(&ct);
+                    tokio::spawn(async move {
+                        ct_2.lock().await.increment();
+                        handle_conn_incoming(conn_id, state, t_read, tenant_tx, req_tx).await;
+                        ct_2.lock().await.decrement();
+                        debug!("<Client @ {:?}> Disconnected", conn_id);
+                    });
+                }
+                Err(x) => {
+                    error!("Listener failed: {}", x);
+                    break;
+                }
+            }}
+            _ = notify.notified() => {
+                warn!("Reached shutdown timeout, so terminating");
+                break;
             }
-        };
-        let (t_read, t_write) = transport.into_split();
-
-        // Used to alert our response task of the connection's tenant name
-        // based on the first
-        let (tenant_tx, tenant_rx) = oneshot::channel();
-
-        // Create a state we use to keep track of connection-specific data
-        debug!("<Client @ {}> Initializing internal state", conn_id);
-        let state = Arc::new(Mutex::new(ConnState::default()));
-
-        // Spawn task to continually receive responses from the client that
-        // may or may not be relevant to the connection, which will filter
-        // by tenant and then along any response that matches
-        let res_rx = broadcaster.subscribe();
-        let state_2 = Arc::clone(&state);
-        tokio::spawn(async move {
-            handle_conn_outgoing(conn_id, state_2, t_write, tenant_rx, res_rx).await;
-        });
-
-        // Spawn task to continually read requests from connection and forward
-        // them along to be sent via the client
-        let req_tx = req_tx.clone();
-        tokio::spawn(async move {
-            handle_conn_incoming(conn_id, state, t_read, tenant_tx, req_tx).await;
-        });
+        }
     }
 
     Ok(())
